/**
 * Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 */
import com.oracle.bmc.Region;
import com.oracle.bmc.auth.FileBasedX509CertificateSupplier;
import com.oracle.bmc.auth.S2SAuthenticationDetailsProvider;
import com.oracle.bmc.auth.X509CertificateSupplier;
import com.oracle.bmc.core.Compute;
import com.oracle.bmc.core.ComputeClient;
import com.oracle.bmc.core.requests.ListInstancesRequest;
import com.oracle.bmc.core.responses.ListInstancesResponse;
import com.oracle.bmc.http.CrossTenancyRequestConfigurator;

import java.io.File;
import java.util.HashSet;
import java.util.Set;

public class ExampleCrossTenancyUsage {

    public static void main(String[] args) throws Exception {

        // required, fill in S2S args
        String federationEndpoint = null; // ex, "https://auth.us-phoenix-1.oraclecloud.com"
        String tenancyId = null;
        File certificatePath = null;
        File privateKeyPath = null;

        String targetTenancyId = null; // the other tenancyId, the one owning the resource

        // The value associated with the header should be a list of tenancy OCIDs that might be accessed by this
        // request. For this release, two tenancy OCIDs should be passed in the header:
        // 1.  Target tenancy OCID (e.g. the customer tenancy)
        // 2.  Requesting tenancy OCID (the tenancy trying to access the customer's tenancy)
        String[] authorizedTenancyIds =
                new String[] {targetTenancyId, tenancyId}; // set appropriate authorized IDs

        // passphrase is optional
        char[] privateKeyPassphrase = null;

        // optional, add intermediary suppliers to the set if needed
        Set<X509CertificateSupplier> intermediaryCertificateSuppliers = new HashSet<>();

        // required, fill in request args
        String compartmentId = null;

        X509CertificateSupplier leafCertificateSupplier =
                new FileBasedX509CertificateSupplier(
                        certificatePath, privateKeyPath, privateKeyPassphrase);
        S2SAuthenticationDetailsProvider authenticationDetailsProvider =
                S2SAuthenticationDetailsProvider.builder()
                        .intermediateCertificateSuppliers(intermediaryCertificateSuppliers)
                        .build(federationEndpoint, tenancyId, leafCertificateSupplier);

        Compute computeClient =
                new ComputeClient(
                        authenticationDetailsProvider,
                        null, // use default config
                        new CrossTenancyRequestConfigurator(authorizedTenancyIds));
        computeClient.setRegion(Region.US_PHOENIX_1);
        ListInstancesResponse instances =
                computeClient.listInstances(
                        ListInstancesRequest.builder().compartmentId(compartmentId).build());
    }
}
